@@ -17,7 +17,7 @@ def run(meter, duration):
         read [duration] seconds of data
         and return prep using identity reconstruction matrix"""
     # figure out how many rows of data to capture
-    nrows = (duration+2) * 3000  # noncontact sample frequency
+    nrows = (duration + 2) * 3000  # noncontact sample frequency
 
     num_sensors = len(meter['sensors']['current']['sensor_indices'])
 
@@ -42,6 +42,7 @@ def run(meter, duration):
     """
 
     # capture reader
+    tasks = []
     capture_args = argparse.Namespace(**{
         "meter_type": "noncontact",
         "align": False,
@@ -51,7 +52,7 @@ def run(meter, duration):
     })
 
     my_capture = capture.Capture()
-    capture_future = asyncio.ensure_future(
+    tasks.append(
         my_capture.run(capture_args, npipe_raw))
 
     # reconstructor (needed to integrate voltage signal)
@@ -64,7 +65,7 @@ def run(meter, duration):
         'voltage_matrix': [1.0],
     }
     my_reconstructor = reconstructor.Reconstructor()
-    asyncio.ensure_future(
+    tasks.append(
         my_reconstructor.run(
             argparse.Namespace(**reconstructor_configs),
             {'raw': npipe_raw},
@@ -79,7 +80,7 @@ def run(meter, duration):
         'min_amp': 10
     }
     my_sinefit = sinefit.Sinefit()
-    asyncio.ensure_future(
+    tasks.append(
         my_sinefit.run(
             argparse.Namespace(**sinefit_configs),
             {'iv': npipe_iv},
@@ -97,14 +98,17 @@ def run(meter, duration):
     }
 
     my_prep = prep.Prep()
-
-    loop = asyncio.get_event_loop()
-
-    loop.run_until_complete(my_prep.run(
+    tasks.append(
+        my_prep.run(
             argparse.Namespace(**prep_configs),
             {'iv': npipe_iv2,
              'zero_crossings': npipe_zc},
             {'prep': npipe_prep}))
+
+    loop = asyncio.get_event_loop()
+
+    loop.run_until_complete(asyncio.gather(*tasks))
+
     prep_data = npipe_prep.read_nowait()
 
     """
@@ -144,4 +148,4 @@ def run(meter, duration):
     plt.show()
     """
 
-    return prep_data['data'][:duration*60]
+    return prep_data['data'][:duration * 60]
