@@ -8,22 +8,23 @@ import asyncio
 import argparse
 import os
 import subprocess
-import time
 import numpy as np
-import threading
 import warnings
-
-import matplotlib
-matplotlib.use('tkagg')
-from matplotlib import pyplot as plt
+import time
 
 from cliff.command import Command
 
+import matplotlib
+from matplotlib import pyplot as plt
+
 from joule import LocalPipe
-from nilm.readers import capture
+from joule.models.pipes import EmptyPipe
+from nilm.readers import capture, serialcapture
 from nilm.meter import meter
 from nilm.meter.utils import MeterConfigurationError
+import signal
 
+matplotlib.use('tkagg')
 
 METERS_FILE = "/opt/configs/meters.yml"
 METER_CAL = "/opt/configs/meters"
@@ -48,8 +49,8 @@ PLOT_WIDTH = 200
 
 
 class ScopeCmd(Command):
-    "Realtime scope for contact and non-contact meters"
-    
+    """Realtime scope for contact and non-contact meters"""
+
     def get_parser(self, prog_name):
         parser = super(ScopeCmd, self).get_parser(prog_name)
         # meter
@@ -81,7 +82,7 @@ class ScopeCmd(Command):
         # Stop the jouled service if it is running
         ret = subprocess.call(["pgrep", "jouled"],
                               stdout=FNULL, stderr=FNULL)
-        if(ret == 0):
+        if ret == 0:
             subprocess.call(["sudo", "service", "jouled", "stop"],
                             stdout=FNULL, stderr=FNULL)
             time.sleep(5)
@@ -91,8 +92,8 @@ class ScopeCmd(Command):
             restart_capture = False
 
         # pipes to receive data
-        if(my_meter['type'] == 'contact'):
-            npipe = LocalPipe("uint16_6",name="raw")
+        if my_meter['type'] == 'contact':
+            npipe = LocalPipe("uint16_6", name="raw")
         else:
             npipe = LocalPipe("int16_8", name="raw")
 
@@ -103,17 +104,18 @@ class ScopeCmd(Command):
             "max_gap": 100,  # ignored
             "nrows": 0  # run forever
         }
-        if(my_meter["type"] == "noncontact"):
+        if my_meter["type"] == "noncontact":
             config["tty"] = my_meter["tty"]
         else:
             config["ip_address"] = my_meter["ip_address"]
         capture_args = argparse.Namespace(**config)
-        my_capture = capture.Capture()
+        self.my_capture = capture.Capture()
 
         # setup the plot
         self.plot_width = CONTACT_NUM_POINTS
-        if(my_meter['type'] == TYPE_NON_CONTACT):
+        if my_meter['type'] == TYPE_NON_CONTACT:
             self.plot_width = NON_CONTACT_NUM_POINTS
+            serialcapture.BLOCK_SIZE = 1000*serialcapture.ROW_BYTES
 
         # suppress the rcParam warning
         warnings.filterwarnings("ignore",
@@ -121,69 +123,65 @@ class ScopeCmd(Command):
                                 module='matplotlib')
         (plot, fig, data, ax, bkgd) = self.setup_plot(my_meter, channels)
         fig.canvas.mpl_connect('close_event', self.shutdown)
-        # setup the tasks and run the loop
-        tasks = [
-            asyncio.ensure_future(
-                my_capture.run(capture_args, npipe)),
-            asyncio.ensure_future(self.update_plot(
-                npipe, channels, fig, data, ax, bkgd))]
-        loop = asyncio.get_event_loop()
-        self.shutdown = False
-        t = threading.Thread(target=self.run_loop, args=(loop, tasks))
-        t.start()
+
+        # loop = asyncio.get_event_loop()
+        self.stop_requested = False
+        plot.ion()
         plot.show()
-        
-        self.shutdown = True
-        loop.call_soon_threadsafe(my_capture.stop)
-        t.join(timeout=5.0)
-        
-    def run_loop(self, loop, tasks):
-        asyncio.set_event_loop(loop)
-        loop.run_until_complete(asyncio.gather(*tasks))
+        loop = asyncio.get_event_loop()
+        loop.add_signal_handler(signal.SIGINT, self.shutdown)
+        print("Type Ctrl-C to exit")
+        loop.run_until_complete(asyncio.gather(
+            self.my_capture.run(capture_args, npipe),
+            self.update_plot(npipe, channels, fig, data, ax, bkgd)
+        ))
         loop.close()
-        print("done with loop")
-        
-    def shutdown(self, event):
-        pass
+        # loop.call_soon_threadsafe(my_capture.stop)
+
+    def shutdown(self):
+        self.my_capture.stop()
+        self.stop_requested = True
 
     async def update_plot(self, npipe, channels, fig, data, ax, bkgd):
         dead_loops = 0
         MAX_DEAD_LOOPS = 10
-        while(not self.shutdown):
-            sensor_data = await npipe.read(flatten=True)
-            if(len(sensor_data) < self.plot_width):
-                await asyncio.sleep(.1)
+        while not self.stop_requested:
+            try:
+                sensor_data = await npipe.read(flatten=True)
+            except EmptyPipe:
+                break
+            if len(sensor_data) < self.plot_width:
+                await asyncio.sleep(.01)
                 dead_loops += 1
-                if(dead_loops > MAX_DEAD_LOOPS):
+                if dead_loops > MAX_DEAD_LOOPS:
                     break  # the producer is not providing data
                 continue  # not enough data to plot
             dead_loops = 0
             sensor_data = sensor_data - np.mean(sensor_data, axis=0)
- #           fig.canvas.restore_region(bkgd)
+            #           fig.canvas.restore_region(bkgd)
             for i in range(len(channels)):
                 points = data[i]
-                idx = channels[i]+1  # ts is column 0
+                idx = channels[i] + 1  # ts is column 0
                 cur_data = points.get_data()
                 points.set_data(cur_data[0],
                                 sensor_data[:self.plot_width, idx])
-#                ax.draw_artist(points)
-#            fig.canvas.blit(ax.bbox)
+            #                ax.draw_artist(points)
+            #            fig.canvas.blit(ax.bbox)
             fig.canvas.draw()
-            consumed_data = len(sensor_data)-(len(sensor_data) % self.plot_width)
+            consumed_data = len(sensor_data) - (len(sensor_data) % self.plot_width)
             npipe.consume(consumed_data)
-            await asyncio.sleep(.1)
-        print("exiting plotting coroutine")
-        
+            await asyncio.sleep(.01)
+
     def setup_plot(self, my_meter, channels):
 
         # remove navigation toolbar
         matplotlib.rcParams['toolbar'] = 'None'
 
         fig, ax = plt.subplots(1, 1)
-        #ax.hold(True)
+        # ax.hold(True)
         ax.grid(True)
 
-        if(my_meter["type"] == TYPE_NON_CONTACT):
+        if my_meter["type"] == TYPE_NON_CONTACT:
             ax.set_ylim([-1000, 1000])
         else:
             ax.set_ylim([-6000, 6000])
@@ -192,7 +190,7 @@ class ScopeCmd(Command):
         ax.set_xticks([])
         background = fig.canvas.copy_from_bbox(ax.bbox)
         plot_data = []
-        if(my_meter["type"] == TYPE_NON_CONTACT):
+        if my_meter["type"] == TYPE_NON_CONTACT:
             ax.set_title("Raw data from non-contact meter [%s]" %
                          my_meter["serial_number"])
             for idx in channels:
@@ -201,7 +199,7 @@ class ScopeCmd(Command):
                     exit(1)
                 # figure out this channel's role
                 role = "[%d] --unused--" % idx
-                if(idx == my_meter["sensors"]["voltage"]["sensor_index"]):
+                if idx == my_meter["sensors"]["voltage"]["sensor_index"]:
                     role = "[%d] Voltage" % idx
                 else:
                     for k in my_meter["sensors"]["current"]["sensor_indices"]:
@@ -216,10 +214,10 @@ class ScopeCmd(Command):
             ax.set_title(
                 "Raw data from contact meter [%s]" % my_meter["ip_address"])
             for idx in channels:
-                if(idx < 0 or idx > 5):
+                if idx < 0 or idx > 5:
                     print("Error, invalid channel: %d" % idx)
                     exit(1)
-                if(idx < 3):
+                if idx < 3:
                     role = "[%d] Current" % idx
                 else:
                     role = "[%d] Voltage" % idx
@@ -229,4 +227,4 @@ class ScopeCmd(Command):
                 plot_data.append(points)
         ax.legend(loc=1)  # upper right
         plt.draw()
-        return (plt, fig, plot_data, ax, background)
+        return plt, fig, plot_data, ax, background
